<?php
// $Id: ec_store.inc,v 1.5.2.3 2009/02/09 18:57:07 recidive Exp $

/**
 * @file
 * Code here is not specific to any process, might be any user, and is not run infrequently.
 */

// -------------------------------------------------------------------------
// Infrequent processing

/**
 * Implementation of hook_cron().
 */
function _ec_store_cron() {
  // Any item whose workflow is 'shipped', move it to 'completed'. This does
  // seem presumptous. Ideally we'd email the user a list of shipped items for
  // day and then adjust the shipping workflow after a given time period, such
  // as 2 week (in case of returns or something) rather than automatically
  // closing it here.
  db_query('UPDATE {ec_transaction} SET workflow = %d
    WHERE workflow = %d AND allocation = %d', EC_WORKFLOW_COMPLETED,
    EC_WORKFLOW_SHIPPED, EC_ALLOC_COMPLETED);

  if (variable_get('payment_notices', 1) and
      (time() >= (variable_get('ecommerce_store_summary_email_sent', 0) +
      variable_get('ecommerce_store_summary_email_frequency', 60 * 60 * 24)))) {
    global $base_url;

    // Find order(s) with a 'received' payment status needs action.
    if ($count = db_result(db_query('SELECT COUNT(st.txnid)
      FROM {ec_transaction} AS st WHERE st.allocation = %d AND
      (st.workflow = %d OR st.workflow = %d OR st.workflow = %d OR st.workflow = %d)',
      EC_ALLOC_COMPLETED, EC_WORKFLOW_RECEIVED, EC_WORKFLOW_INVOICED,
      EC_WORKFLOW_SHIPPED, EC_WORKFLOW_WAITING_RESPONSE))) {

      $beg_month    = mktime(0, 0, 0, date('m'), 1, date('Y'));
      $mon_orders  = db_result(db_query('SELECT COUNT(txnid)
        FROM {ec_transaction} where created >= %d AND allocation = %d AND
        workflow = %d', $beg_month, EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED));
      $mon_sales    = '$'. number_format(db_result(db_query('SELECT SUM(gross)
        FROM {ec_transaction} where created >= %d AND allocation = %d AND
        workflow = %d', $beg_month, EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED)), 2);

      $buf = t('@count order(s) with a "cleared" payment status needs action:',
        array('@count' => $count)) ."\n";
      $result = db_query('SELECT * FROM {ec_transaction} AS st
        WHERE st.allocation = %d AND st.workflow IN (%d, %d, %d, %d)
        ORDER BY st.txnid DESC', EC_ALLOC_COMPLETED, EC_WORKFLOW_RECEIVED,
        EC_WORKFLOW_INVOICED, EC_WORKFLOW_SHIPPED, EC_WORKFLOW_WAITING_RESPONSE);
      while ($data = db_fetch_object($result)) {

        $buf .= t('Order #@txnid created @timestamp ago', array('@txnid' => $data->txnid,
          '@timestamp' => format_interval(time() - $data->created, 2))) ."\n".
          t('Workflow status: @transaction-workflow',
          array('@transaction-workflow' => transaction_get_workflow($data->workflow))) ."\n".
          t('Email: @email', array('@email' => $data->mail)) ."\n".
          t('Total: @gross', array('@gross' => format_currency($data->gross))) ."\n";

        $r = db_query('SELECT stp.*, st.allocation, st.workflow
          FROM {ec_transaction} AS st, {ec_transaction_product} AS stp
          WHERE st.txnid = %d AND st.txnid = stp.txnid
          ORDER BY st.txnid DESC', $data->txnid);
        $items = NULL;
        $has_shippable_item = FALSE;
        while ($item = db_fetch_object($r)) {
          if ($product = ec_product_load($item)) {
            foreach ($product as $key => $value) {
              if ($key != 'title' && $key != 'price') {
                $item->$key = $value;
              }
            }
          }
          if (product_is_shippable($item->vid)) {
            $has_shippable_item = TRUE;
          }

          $sku = ($item->sku) ? " [$item->sku]" : "";
          $items .= t('@order of @title at @price each', array(
            '@order' => format_plural($item->qty, '1 order', '@count orders'),
            '@title' => $item->title . $sku,
            '@price' => format_currency($item->price)));
        }

        // If the transaction has no shippable items and the payment status is
        // 'complete' and the workflow is 'transaction received', set the
        // workflow to 'complete'
        if (empty($has_shippable_item) && $data->allocation == EC_ALLOC_COMPLETED && $data->workflow == EC_WORKFLOW_RECEIVED) {
          db_query('UPDATE {ec_transaction} SET workflow = %d WHERE txnid = %d', EC_WORKFLOW_COMPLETED, $data->txnid);
        }

        $shipping_to = theme('ec_store_format_address', $data, 'shipping', 'text');
        $billing_to = theme('ec_store_format_address', $data, 'billing', 'text');

        $buf .= t("Shipping to:\n!shipping_address", array('!shipping_address' => $shipping_to)) ."\n".
          t("Billing to:\n!billing_address", array('!billing_address' => $billing_to)) ."\n".
          "$items\n".
          t('Update:') ."$base_url/admin/store/transaction/edit/$data->txnid\n".
          "=====\n";

      }

      $output = $buf;
      $output .= t('(These orders will no longer be listed once the transaction workflow is set to either "Completed" or "Canceled")') .
        "\n". t("Monthly Summary\nNumber of completed orders: @monthly-orders\nSales: @monthly-sales",
        array('@monthly-orders' => $mon_orders, '@monthly-sales' => $mon_sales)) ."\n".
        t('Want more reports? Visit @base_url/admin/store', array('@base_url' => $base_url)) ."\n";

      $subject    = t('@site-name transaction summary', array('@site-name' => variable_get('site_name', 'Drupal')));
      $to         = variable_get('site_mail', ini_get('sendmail_from'));
      $from       = variable_get('site_mail', ini_get('sendmail_from'));
      $headers    = array(
        'From'        => $from,
        'Reply-to'    => $from,
        'X-Mailer'    => 'PHP',
        'Return-path' => $from,
        'Errors-to'   => $from,
      );

      // TODO: drupal_mail('store', 'ec_order_summary', $to, $subject, $output, $from, $headers);
      variable_set('ecommerce_store_summary_email_sent', time());
    }
  } // End receive payment notices.
}

/**
 * Saves all addresses in transaction using ec_store_transaction_address_save().
 *
 * @param $txn
 *   Object, transaction with address array.
 */
function ec_store_transaction_addresses_save($txn) {
  $txn = (object)$txn; // just to be sure

  // If we where not passed a valid transaction and we can't save.
  if (!isset($txn->txnid) || !is_array($txn->address)) {
    return FALSE;
  }
  foreach ($txn->address as $type => $address) {
    if (is_array($address)) {
      $address = (object)$address;
    }
    $address->txnid = $txn->txnid;
    ec_store_transaction_address_save($address, $type);
  }
}

/**
 * Saves an individual address to the passed transaction.
 *
 * @param $address
 *   Object, address object.
 * @param $type
 *   String, address type(corresponds to type field in database).
 */
function ec_store_transaction_address_save($address, $type) {
  $address = (object)$address; // just to be sure
  $address->type = $type;

  drupal_write_record('ec_transaction_address', $address, array('txnid', 'type'));
  if (!db_affected_rows()) {
    drupal_write_record('ec_transaction_address', $address);
  }
}

/**
 * Creates a standard address form.
 * When passed a display field we filter out all other sections and display only
 * the one given. If passed shipping only a shipping address will be shown.
 *
 * @param $txn
 *   Objectm this is a transaction object used for default values.
 * @param $display
 *   String, option field that allows the form to filter sections.
 */
function ec_store_transaction_addresses_form($txn, $display = NULL) {
  $form['address']['#tree'] = TRUE;

  $form['address']['shipping'] = ec_store_address_form($txn->address['shipping']);
  $form['address']['shipping']['#type'] = 'fieldset';
  $form['address']['shipping']['#title'] = t('Shipping address');

  $form['address']['billing'] = ec_store_address_form($txn->address['billing']);
  $form['address']['billing']['#type'] = 'fieldset';
  $form['address']['billing']['#title'] = t('Billing address');

  switch ($display) {
    case 'shipping':
      unset($form['billing']);
      break;

    case 'billing':
      unset($form['shipping']);
      break;
  }
  return $form;
}

function ec_store_address_form($address) {
  module_load_include('inc', 'ec_store', 'ec_store.localization');

  if (isset($address->aid)) {
    $form['type'] = array(
      '#type'  => 'hidden',
      '#value' => $address->type,
    );
  }

  $form['firstname'] = array(
    '#type'          => 'textfield',
    '#title'         => t('First name'),
    '#default_value' => $address->firstname,
    '#size'          => 32,
    '#maxlength'     => 32,
  );
  $form['lastname'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Last name'),
    '#default_value' => $address->lastname,
    '#size'          => 32,
    '#maxlength'     => 32,
  );
  $form['country'] = array(
    '#type'          => 'select',
    '#title'         => t('Country'),
    '#default_value' => ($address->country ? $address->country : variable_get('ec_country', '')),
    '#options'       => _ec_store_location_countries(),
  );
  $form['street1'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Street'),
    '#default_value' => $address->street1,
    '#size'          => 60,
    '#maxlength'     => 64,
  );
  $form['street2'] = array(
    '#type'          => 'textfield',
    '#default_value' => $address->street2,
    '#size'          => 60,
    '#maxlength'     => 64,
  );
  $form['city'] = array(
    '#type'          => 'textfield',
    '#title'         => t('City'),
    '#default_value' => $address->city,
    '#size'          => 32,
    '#maxlength'     => 32,
  );
  $form['state'] = array(
    '#type'          => 'textfield',
    '#title'         => t('State/Province'),
    '#default_value' => $address->state,
    '#size'          => 32,
    '#maxlength'     => 32,
  );
  $form['zip'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Zip/Postal Code'),
    '#default_value' => $address->zip,
    '#size'          => 10,
    '#maxlength'     => 10,
  );
  return $form;
}
